# --- BEGIN COPYRIGHT BLOCK ---
# Copyright (C) 2018 Red Hat, Inc.
# All rights reserved.
#
# License: GPL (version 3 or any later version).
# See LICENSE for details.
# --- END COPYRIGHT BLOCK ---
#

import ldap
import pytest
import logging
from lib389.topologies import topology_st
from lib389._constants import PASSWORD, DEFAULT_SUFFIX

from lib389.idm.user import UserAccounts, TEST_USER_PROPERTIES

logging.getLogger(__name__).setLevel(logging.DEBUG)
log = logging.getLogger(__name__)

def test_invalid_hash_fails(topology_st):
    """When given a malformed hash from userpassword migration
    slapi_ct_memcmp would check only to the length of the shorter
    field. This affects some values where it would ONLY verify
    the salt is valid, and thus would allow any password to bind.

    :id: 8131c029-7147-47db-8d03-ec5db2a01cfb
    :setup: Standalone Instance
    :steps:
        1. Create a user
        2. Add an invalid password hash (truncated)
        3. Attempt to bind
    :expectedresults:
        1. User is added
        2. Invalid pw hash is added
        3. Bind fails
    """
    log.info("Running invalid hash test")

    # Allow setting raw password hashes for migration.
    topology_st.standalone.config.set('nsslapd-allow-hashed-passwords', 'on')

    users = UserAccounts(topology_st.standalone, DEFAULT_SUFFIX)
    user = users.create(properties=TEST_USER_PROPERTIES)
    user.set('userPassword', '{CRYPT}XX')

    # Attempt to bind. This should fail.
    with pytest.raises(ldap.INVALID_CREDENTIALS):
        user.bind(PASSWORD)
    with pytest.raises(ldap.INVALID_CREDENTIALS):
        user.bind('XX')
    with pytest.raises(ldap.INVALID_CREDENTIALS):
        user.bind('{CRYPT}XX')

    log.info("PASSED")

